@@ -1,79 +1,79 @@
 #' Import Objects From a Package.
 #'
-#' The \code{import::from} and \code{import::into} functions provide an
-#' alternative way to import objects (e.g. functions) from packages. It is
-#' sometimes preferred over using \code{library} (or \code{require}) which will
-#' import all objects exported by the package. The benefit over \code{obj <-
-#' pkg::obj} is that the imported objects will (by default) be placed in a
-#' separate entry in the search path (which can be specified), rather in the
-#' global/current environment. Also, it is a more succinct way of importing
-#' several objects. Note that the two functions are symmetric, and usage is a
-#' matter of preference and whether specifying the \code{.into} argument is
-#' desired. The function \code{import::here} imports into the current environment.
+#' The `import::from` and `import::into` functions provide an alternative way to
+#' import objects (e.g. functions) from packages. It is sometimes preferred over
+#' using `library` (or `require`) which will import all objects exported by the
+#' package. The benefit over `obj <- pkg::obj` is that the imported objects will
+#' (by default) be placed in a separate entry in the search path (which can be
+#' specified), rather in the global/current environment. Also, it is a more
+#' succinct way of importing several objects. Note that the two functions are
+#' symmetric, and usage is a matter of preference and whether specifying the
+#' `.into` argument is desired. The function `import::here` imports into the
+#' current environment.
 #'
-#' The function arguments can be quoted or unquoted as with e.g. \code{library}.
-#' In any case, the character representation is used when unquoted arguments are
+#' The function arguments can be quoted or unquoted as with e.g. `library`. In
+#' any case, the character representation is used when unquoted arguments are
 #' provided (and not the value of objects with matching names). The period in
-#' the argument names \code{.into} and \code{.from} are there to avoid name
-#' clash with package objects. However, while importing of hidden objects (those with
-#' names prefixed by a period) is supported, care should be taken not to conflict
-#' with the argument names. The double-colon syntax \code{import::from}
-#' allows for imports of exported objects (and lazy data) only. To import
-#' objects that are not exported, use triple-colon syntax, e.g.
-#' \code{import:::from}. The two ways of calling the \code{import} functions
-#' analogue the \code{::} and \code{:::} operators themselves.
+#' the argument names `.into` and `.from` are there to avoid name clash with
+#' package objects. However, while importing of hidden objects (those with names
+#' prefixed by a period) is supported, care should be taken not to conflict with
+#' the argument names. The double-colon syntax `import::from` allows for imports
+#' of exported objects (and lazy data) only. To import objects that are not
+#' exported, use triple-colon syntax, e.g. `import:::from`. The two ways of
+#' calling the `import` functions analogue the `::` and `:::` operators
+#' themselves.
 #'
-#' Note that the \code{import} functions usually have the (intended) side-effect
-#' of altering the search path, as they (by default) import objects into the
+#' Note that the `import` functions usually have the (intended) side-effect of
+#' altering the search path, as they (by default) import objects into the
 #' "imports" search path entry rather than the global environment.
 #'
-#' The \code{import} package is not meant to be loaded with \code{library} (and
-#' will output a message about this if attached), but rather it is named to make
-#' the function calls expressive without the need to loading before use, i.e. it is
-#' designed to be used explicitly with the \code{::} syntax, e.g.
-#' \code{import::from(pkg, x, y)}.
+#' The `import` package is not meant to be loaded with `library` (and will
+#' output a message about this if attached), but rather it is named to make the
+#' function calls expressive without the need to loading before use, i.e. it is
+#' designed to be used explicitly with the `::` syntax, e.g. `import::from(pkg,
+#' x, y)`.
 #'
 #' @section Packages vs. modules:
-#' \code{import} can either be used to import objects either from R packages or
-#' from \code{R} source files. If the \code{.from} parameter ends with '.R' or
-#' '.r', \code{import} will look for a source file to import from. A source file
-#' in this context is referred to as a \code{module} in the documentation.
+#' `import` can either be used to import objects either from R packages or from
+#' `R` source files. If the `.from` parameter ends with '.R' or '.r', `import`
+#' will look for a source file to import from. A source file in this context is
+#' referred to as a `module` in the documentation.
 #'
 #' @section Package Versions:
-#' With \code{import} you can specify package version requirements. To do this
-#' add a requirement in parentheses to the package name (which then needs to
-#' be quoted), e.g \code{import::from("parallel (>= 3.2.0)", ...)}.
-#' You can use the operators \code{<}, \code{>}, \code{<=}, \code{>=},
-#' \code{==}, \code{!=}. Whitespace in the specification is irrelevant.
+#' With `import` you can specify package version requirements. To do this add a
+#' requirement in parentheses to the package name (which then needs to be
+#' quoted), e.g `import::from("parallel (>= 3.2.0)", ...)`. You can use the
+#' operators `<`, `>`, `<=`, `>=`, `==`, `!=`. Whitespace in the specification
+#' is irrelevant.
 #'
 #' @rdname importfunctions
 #' @param .from The package from which to import.
-#' @param ... Names or name-value pairs specifying objects to import.
-#'   If arguments are named, then the imported object will have this new name.
-#' @param .into The name of the search path entry, either as a character string
-#'   or as a symbol containing a string. Enclosing the value in curly
-#'   brackets causes the parameter to be treated as an actual
-#'   environment value, rather than the name of an environment. Using
-#'   \code{.into={environment()}} causes imports to be made into the current
-#'   environment; \code{.into=""} is an equivalent shorthand value.
+#' @param ... Names or name-value pairs specifying objects to import. If
+#'   arguments are named, then the imported object will have this new name.
+#' @param .into The environment into which the imported objects should be
+#'   assigned. If the value is of mode `character`, it is treated as referring
+#'   to a named environment on the search path. If it is of mode `environment`,
+#'   the objects are assigned directly to that environment. Using
+#'   `.into=environment()` causes imports to be made into the current
+#'   environment; `.into=""` is an equivalent shorthand value.
 #' @param .library character specifying the library to use when importing from
-#'   packages. Defaults to the latest specified library.
+#'   packages. Defaults to the current set of library paths (note that the
+#'   default value was different in versions up to and including `1.3.0`).
 #' @param .directory character specifying the directory to use when importing
 #'   from modules. Defaults to the current working directory. If .from is a
-#'   module specified using an absolute path (i.e. starting with \code{/}),
-#'   this parameter is ignored.
-#' @param .all logical specifying whether all available objects in a
-#'   package or module should  be imported. It defaults to FALSE unless
-#'   .exclude is being used to omit particular functions.
-#' @param .except character vector specifying any objects that should
-#'   not be imported. Any values specified here override both values
-#'   provided in \code{...} and objects included because of the
-#'   \code{.all} parameter
+#'   module specified using an absolute path (i.e. starting with `/`), this
+#'   parameter is ignored.
+#' @param .all logical specifying whether all available objects in a package or
+#'   module should  be imported. It defaults to FALSE unless .exclude is being
+#'   used to omit particular functions.
+#' @param .except character vector specifying any objects that should not be
+#'   imported. Any values specified here override both values provided in `...`
+#'   and objects included because of the `.all` parameter
 #' @param .chdir logical specifying whether to change directories before
 #'   sourcing a module (this parameter is ignored for libraries)
-#' @param .character_only A logical indicating whether \code{.from} and
-#'   \code{...} can be assumed to be character strings. (Note that this
-#'   parameter does not apply to how the \code{.into} parameter is handled).
+#' @param .character_only A logical indicating whether `.from` and `...` can be
+#'   assumed to be character strings. (Note that this parameter does not apply
+#'   to how the `.into` parameter is handled).
 #' @param .S3 `r badge("experimental")` A logical indicating whether an
 #'   automatic detection and registration of S3 methods should be performed. The
 #'   S3 methods are assumed to be in the standard form `generic.class`. Methods
@@ -92,14 +92,13 @@
 #'
 #' @seealso
 #'   Helpful links:
-#'     \itemize{
-#'       \item{[https://import.rticulate.org](https://import.rticulate.org)}
-#'       \item{[https://github.com/rticulate/import](https://github.com/rticulate/import)}
-#'       \item{[https://github.com/rticulate/import/issues](https://github.com/rticulate/import/issues)}
-#'     }
+#'    * [https://import.rticulate.org](https://import.rticulate.org)
+#'    * [https://github.com/rticulate/import](https://github.com/rticulate/import)
+#'    * [https://github.com/rticulate/import/issues](https://github.com/rticulate/import/issues)
+#'
 #' @md
 from <- function(.from, ..., .into = "imports",
-                 .library = .libPaths()[1L], .directory=".",
+                 .library = .libPaths(), .directory=".",
                  .all=(length(.except) > 0), .except=character(),
                  .chdir = TRUE, .character_only = FALSE, .S3 = FALSE)
 {
@@ -184,7 +183,13 @@ from <- function(.from, ..., .into = "imports",
 
       # Source the file into the new environment.
       packages_before <- .packages()
-      suppress_output(sys.source(file_path(.directory, .from), scripts[[.from]], chdir = .chdir))
+      tryCatch(
+        expr = suppress_output(sys.source(file_path(.directory, .from), scripts[[.from]], chdir = .chdir)),
+        error = function(cnd) {
+          rm(list = .from, envir = scripts)
+          stop("Failed to import ", .from, " due to the following error:\n", cnd$message, call. = FALSE)
+        }
+      )
 
       # If sourcing the script loaded new packages, raise error
       packages_after <- .packages()
@@ -214,19 +219,19 @@ from <- function(.from, ..., .into = "imports",
   } else {
     # Load the package namespace, which is passed to the import calls.
     spec <- package_specs(.from)
-    all_objects <- getNamespaceExports(spec$pkg)
     pkg <- tryCatch(
       loadNamespace(spec$pkg, lib.loc = .library,
                     versionCheck = spec$version_check),
       error = function(e) stop(conditionMessage(e), call. = FALSE)
     )
+    all_objects <- getNamespaceExports(spec$pkg)
     pkg_name <- spec$pkg
   }
   # If .all parameter was specified, override with list of all objects
   # (excluding internal variable __last_modified__)
   # Take care not to lose the names of any manually specified parameters
   if (.all) {
-    all_objects <- setdiff(all_objects, "__last_modified__")
+    all_objects <- setdiff(all_objects, c("__last_modified__", ".packageName"))
     names(all_objects) <- all_objects
     symbols <- c(symbols,all_objects)
     symbols <- symbols[!duplicated(symbols)]
